@@ -62,10 +62,10 @@ from django.http import Http404, HttpResponse
 from django.shortcuts import render_to_response
 from django.template.context import RequestContext, Context
 from django.template.defaultfilters import date, timesince
-from django.template.loader import render_to_string, get_template
+from django.template.loader import get_template
 from django.utils import six
 from django.utils.cache import patch_cache_control
-from django.utils.html import escape
+from django.utils.html import escape, format_html
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 
@@ -76,13 +76,10 @@ except ImportError:
     class SiteProfileNotAvailable(Exception):
         pass
 
-try:
-    from django.template import engines as template_engines
-except ImportError:
-    # Django < 1.8
-    template_engines = None
-
+from djblets.template.context import get_default_template_context_processors
 from djblets.db.query import chainable_select_related_queryset
+from djblets.util.compat.django.template.loader import (render_template,
+                                                        render_to_string)
 from djblets.util.decorators import cached_property
 from djblets.util.http import get_url_params_except
 
@@ -129,85 +126,92 @@ class Column(object):
                  image_height=None, image_alt="", shrink=False, expand=False,
                  sortable=False,
                  default_sort_dir=SORT_DESCENDING, link=False,
-                 link_func=None, cell_clickable=False, css_class=""):
+                 link_func=None, link_css_class=None, cell_clickable=False,
+                 css_class=""):
         """Initialize the column.
 
         When initializing a column as part of a :py:class:`DataGrid` subclass,
         a number of options can be provided.
 
         Args:
-            id (unicode):
+            id (unicode, optional):
                 The unique ID of the column on the datagrid.
 
-            label (unicode):
+            label (unicode, optional):
                 The label to show in the column header.
 
-            detailed_label (unicode):
+            detailed_label (unicode, optional):
                 A detailed label to display in the column customization
                 menu. Defaults to ``label``.
 
-            detailed_label_html (unicode):
+            detailed_label_html (unicode, optional):
                 A detailed label in HTML form to display in the column
                 customization menu. This takes precedence over
                 ``detailed_label``.
 
-            field_name (unicode):
+            field_name (unicode, optional):
                 The name of the field on the model containing the data to
                 render.
 
-            db_field (unicode):
+            db_field (unicode, optional):
                 The name of the database field containing the field used
                 for sorting. Defaults to ``field_name``.
 
-            image_url (unicode):
+            image_url (unicode, optional):
                 The URL to the image used in the header and navigation menu.
                 This cannot be used with ``image_class``.
 
-            image_class (unicode):
+            image_class (unicode, optional):
                 The CSS class of a spritesheet icon to use in the header
                 and navigation menu. This cannot be used with ``image_url``.
 
-            image_width (int):
+            image_width (int, optional):
                 The width of the image.
 
-            image_height (int):
+            image_height (int, optional):
                 The height of the image.
 
-            image_alt (unicode):
+            image_alt (unicode, optional):
                 The alt text for the image.
 
-            shrink (bool):
+            shrink (bool, optional):
                 If ``True``, the column's width will be calculated to its
                 minimum size.
 
-            expand (bool):
+            expand (bool, optional):
                 If ``True``, the column's width will be calculated to its
                 maximum size. If there are other expanded columns, they'll
                 share the available width equally.
 
-            sortable (bool):
+            sortable (bool, optional):
                 If ``True``, the column can be sorted. This requires a
                 ``db_field`` that allows for sorting.
 
-            default_sort_dir (int):
+            default_sort_dir (int, optional):
                 The default sorting direction when the user activates sorting.
                 Either :py:attr:`SORT_DESCENDING`
                 or :py:attr:`SORT_ASCENDING`.
 
-            link (bool):
+            link (bool, optional):
                 If ``True``, the contents will be linked to the URL
                 returned by ``link_func`` or
                 :py:meth:`DataGrid.link_to_object`.
 
-            link_func (callable):
+            link_func (callable, optional):
                 Optional function that returns a URL for the link.
 
-            cell_clickable (bool):
+            link_css_class (unicode or callable, optional):
+                The CSS class or classes to define on ``<a>`` for the link
+                for the cell, if setting ``link=True``. This can be a
+                function returning the classes.
+
+            cell_clickable (bool, optional):
                 If ``True``, clicking anywhere on the cell will navigate to
                 the URL defined, if any.
 
-            css_class (unicode):
-                The CSS class or classes to define on the cell.
+            css_class (unicode, optional):
+                The CSS class or classes to define on the cell. This can be
+                a function returning the classes.
         """
         assert not (image_class and image_url)
 
@@ -231,6 +235,7 @@ class Column(object):
         self.link_func = (
             link_func or
             (lambda state, x, y: state.datagrid.link_to_object(state, x, y)))
+        self.link_css_class = link_css_class
         self.css_class = css_class
 
         self.cell_template = None
@@ -376,7 +381,7 @@ class Column(object):
             unsort_url = url_prefix + ','.join(sort_list[1:])
             sort_url = url_prefix + ','.join(sort_list)
 
-        ctx = Context({
+        return render_template(datagrid.column_header_template_obj, {
             'column': self,
             'column_state': state,
             'in_sort': in_sort,
@@ -386,8 +391,6 @@ class Column(object):
             'unsort_url': unsort_url,
         })
 
-        return mark_safe(datagrid.column_header_template_obj.render(ctx))
-
     def collect_objects(self, state, object_list):
         """Iterate through the objects and builds a cache of data to display.
 
@@ -453,20 +456,21 @@ class Column(object):
             logger.exception('Error when calling render_data for DataGrid '
                              'Column %r: %s',
                              self, e)
-            rendered_data = None
-
-        if render_context:
-            url = render_context.get('_datagrid_object_url')
-        else:
-            url = None
+            rendered_data = ''
 
+        # We use empty strings instead of None just to keep cache keys small.
+        url = ''
         css_class = ''
+        link_css_class = ''
 
         if self.link:
-            try:
-                url = self.link_func(state, obj, rendered_data)
-            except AttributeError:
-                pass
+            if self.link_func:
+                try:
+                    url = self.link_func(state, obj, rendered_data)
+                except AttributeError:
+                    pass
+            elif render_context:
+                url = render_context.get('_datagrid_object_url')
 
         if self.css_class:
             if six.callable(self.css_class):
@@ -474,14 +478,29 @@ class Column(object):
             else:
                 css_class = self.css_class
 
-        key = "%s:%s:%s:%s" % (state.last, rendered_data, url, css_class)
+        if self.link_css_class:
+            if six.callable(self.link_css_class):
+                link_css_class = self.link_css_class(obj)
+            else:
+                link_css_class = self.link_css_class
+
+        key = '%s:%s:%s:%s:%s' % (state.last, rendered_data, url, css_class,
+                                  link_css_class)
 
         if key not in state.cell_render_cache:
-            ctx = Context(render_context)
+            if url:
+                css_class = '%s has-link' % css_class
+
+            ctx = {}
+
+            if render_context:
+                ctx.update(render_context)
+
             ctx.update({
                 'column': self,
                 'column_state': state,
-                'css_class': css_class,
+                'css_class': css_class.strip(),
+                'link_css_class': link_css_class,
                 'url': url,
                 'data': mark_safe(rendered_data)
             })
@@ -491,7 +510,7 @@ class Column(object):
             if template is None:
                 template = state.datagrid.cell_template_obj
 
-            state.cell_render_cache[key] = mark_safe(template.render(ctx))
+            state.cell_render_cache[key] = render_template(template, ctx)
 
         return state.cell_render_cache[key]
 
@@ -597,7 +616,7 @@ class StatefulColumn(object):
         try:
             column.setup_state(self)
         except Exception as e:
-            logger.exception('Error when calling setup_state in DataGrid '
+            logger.exception('Error when calling setup_state for DataGrid '
                              'Column %r: %s',
                              self.column, e)
 
@@ -680,14 +699,14 @@ class CheckboxColumn(Column):
         """
         super(CheckboxColumn, self).__init__(
             shrink=shrink,
-            label=mark_safe(
+            label=format_html(
                 '<input class="datagrid-header-checkbox"'
-                ' type="checkbox" data-checkbox-name="%s" />'
-                % checkbox_name),
+                ' type="checkbox" data-checkbox-name="{0}" />',
+                checkbox_name),
             detailed_label=detailed_label,
-            detailed_label_html=mark_safe(
-                '<input type="checkbox" /> %s'
-                % detailed_label),
+            detailed_label_html=format_html(
+                '<input type="checkbox" /> {0}',
+                detailed_label),
             *args, **kwargs)
 
         self.show_checkbox_header = show_checkbox_header
@@ -699,11 +718,11 @@ class CheckboxColumn(Column):
             checked = ''
 
             if self.is_selected(state, obj):
-                checked = 'checked="true"'
+                checked = mark_safe('checked="true"')
 
-            return ('<input type="checkbox" data-object-id="%s" '
-                    'data-checkbox-name="%s" %s />'
-                    % (obj.pk, escape(self.checkbox_name), checked))
+            return format_html('<input type="checkbox" data-object-id="{0}" '
+                               'data-checkbox-name="{1}" {2} />',
+                               obj.pk, self.checkbox_name, checked)
         else:
             return ''
 
@@ -1069,24 +1088,24 @@ class DataGrid(object):
         # Figure out the columns we're going to display
         # We're also going to calculate the column widths based on the
         # shrink and expand values.
-        colnames_str = self.request.GET.get('columns', profile_columns_list)
+        colnames = self.request.GET.get('columns', profile_columns_list) or ''
 
-        if colnames_str:
-            colnames = colnames_str.split(',')
-        else:
-            colnames = self.default_columns
-            colnames_str = ",".join(colnames)
+        columns = filter(None, [
+            self.get_column(colname)
+            for colname in colnames.split(',')
+        ])
+
+        if not columns:
+            colnames = ','.join(self.default_columns)
+            columns = [
+                self.get_column(colname)
+                for colname in self.default_columns
+            ]
 
         expand_columns = []
         normal_columns = []
 
-        for colname in colnames:
-            column_def = self.get_column(colname)
-
-            if not column_def:
-                # The user specified a column that doesn't exist. Skip it.
-                continue
-
+        for column_def in columns:
             column = self.get_stateful_column(column_def)
             self.columns.append(column)
             column.active = True
@@ -1131,11 +1150,19 @@ class DataGrid(object):
         sort_str = self.request.GET.get('sort', profile_sort_list)
 
         if sort_str:
-            self.sort_list = [
-                sort_column
-                for sort_column in sort_str.split(',')
-                if sort_column
-            ]
+            self.sort_list = []
+
+            for sort_item in sort_str.split(','):
+                if sort_item[0] == '-':
+                    base_sort_item = sort_item[1:]
+                else:
+                    base_sort_item = sort_item
+
+                column = self.get_column(base_sort_item)
+
+                if column and column.sortable:
+                    self.sort_list.append(sort_item)
+
         else:
             self.sort_list = self.default_sort
             sort_str = ",".join(self.sort_list)
@@ -1149,8 +1176,8 @@ class DataGrid(object):
         # settings back to the profile.
         if profile:
             if (self.profile_columns_field and
-                    colnames_str != profile_columns_list):
-                setattr(profile, self.profile_columns_field, colnames_str)
+                colnames != profile_columns_list):
+                setattr(profile, self.profile_columns_field, colnames)
                 profile_dirty = True
 
             if self.profile_sort_field and sort_str != profile_sort_list:
@@ -1216,20 +1243,28 @@ class DataGrid(object):
 
         # Generate the actual list of fields we'll be sorting by
         sort_list = []
+
         for sort_item in self.sort_list:
-            if sort_item[0] == "-":
+            if sort_item[0] == '-':
                 base_sort_item = sort_item[1:]
-                prefix = "-"
+                prefix = '-'
             else:
                 base_sort_item = sort_item
-                prefix = ""
+                prefix = ''
 
             if sort_item:
                 column = self.get_column(base_sort_item)
                 if not column:
-                    logger.warning('Skipping non-existing sort column "%s" '
-                                   'for user "%s".',
-                                   base_sort_item, self.request.user.username)
+                    logger.warning('Skipping non-existing sort column "%s"',
+                                   base_sort_item,
+                                   request=self.request)
+                    continue
+
+                elif not column.sortable:
+                    logger.warning('Skipping column "%s" which is not '
+                                   'sortable',
+                                   base_sort_item,
+                                   request=self.request.user.username)
                     continue
 
                 stateful_column = self.get_stateful_column(column)
@@ -1240,10 +1275,12 @@ class DataGrid(object):
                     except Exception as e:
                         logger.exception('Error when calling get_sort_field '
                                          'for DataGrid Column %r: %s',
-                                         column, e)
-                        sort_field = ''
+                                         column, e,
+                                         request=self.request)
+                        continue
 
-                    sort_list.append(prefix + sort_field)
+                    if sort_field:
+                        sort_list.append(prefix + sort_field)
 
                     # Lookups spanning tables require that we query from those
                     # tables. In order to keep things simple, we'll just use
@@ -1339,7 +1376,8 @@ class DataGrid(object):
                 self.rows.append({
                     'object': obj,
                     'cells': [column.render_cell(obj, render_context)
-                              for column in self.columns]
+                              for column in self.columns],
+                    'url': obj_url,
                 })
         except Exception as e:
             logger.exception('Error when calling render_cell for DataGrid '
@@ -1398,14 +1436,13 @@ class DataGrid(object):
             context.update(self.extra_context)
             context.update(render_context)
 
-            return mark_safe(render_to_string(self.listview_template,
-                                              Context(context)))
+            return render_to_string(self.listview_template, context)
         except Exception:
             trace = traceback.format_exc()
             logger.exception('Failed to render datagrid:\n%s',
                              trace,
                              request=self.request)
-            return mark_safe('<pre>%s</pre>' % trace)
+            return format_html('<pre>{0}</pre>', trace)
 
     def render_listview_to_response(self, request=None, render_context=None):
         """Render the listview to a response.
@@ -1514,8 +1551,7 @@ class DataGrid(object):
 
         context.update(self.extra_context)
 
-        return mark_safe(render_to_string(self.paginator_template,
-                                          Context(context)))
+        return render_to_string(self.paginator_template, context)
 
     def build_paginator(self, queryset):
         """Build the paginator for the datagrid.
@@ -1541,34 +1577,11 @@ class DataGrid(object):
         RequestContext, but it's possible to build one and then pull out
         the contents into a dictionary.
         """
-        request_context = RequestContext(self.request)
+        request = self.request
         render_context = {}
 
-        if template_engines:
-            # Django >= 1.8
-            #
-            # RequestContext is no longer populated when created, but rather
-            # during template render. It's bound to a template during
-            # the render, at which point all the context processors will load.
-            # We need to simulate this.
-            #
-            # This depends on the 'django' template engine being loaded. Not
-            # all projects will have that, but most will. For now, it's just
-            # a hard requirement.
-            try:
-                template = template_engines['django'].from_string('')
-            except KeyError:
-                raise ImproperlyConfigured(
-                    'The "django" template engine must be defined in order '
-                    'to render datagrids.')
-
-            with request_context.bind_template(template.template):
-                for d in request_context:
-                    render_context.update(d)
-        else:
-            # Django < 1.8
-            for d in request_context:
-                render_context.update(d)
+        for context_processor in get_default_template_context_processors():
+            render_context.update(context_processor(request))
 
         return render_context
 
