@@ -22,7 +22,7 @@ django_version = '>=1.6.11,<1.10.999'
 #: Dependencies required for LessCSS pipelining.
 lesscss_npm_dependencies = {
     'less': '^3.11.0',
-    'less-plugin-autoprefix': '^2.0.0',
+    '@beanbag/less-plugin-autoprefix': '^2.0.1',
 }
 
 #: Dependencies required for UglifyJS JavaScript compression.
@@ -48,9 +48,9 @@ npm_dependencies.update(babel_npm_dependencies)
 package_dependencies = {
     'Django': django_version,
     'django-pipeline': '>=1.6.14,<1.6.999',
-    'dnspython': '>=1.14.0',
-    'feedparser': '>=5.1.2',
-    'pillowfight': '',
+    'dnspython': '>=1.14.0,<1.999',
+    'feedparser': '>=5.1.2,<5.999',
+    'pillowfight': '>=0.4',
     'publicsuffix': '>=1.1',
     'python-dateutil': '>=1.5',
     'pytz': '',
