@@ -59,7 +59,7 @@ PIPELINE = {
         '--no-color',
         '--source-map',
         '--js',
-        '--autoprefix=> 2%, ie >= 9'
+        '--plugin=@beanbag/less-plugin-autoprefix=> 2%, ie >= 9',
     ],
     'UGLIFYJS_BINARY': os.path.join(NODE_PATH, 'uglify-js', 'bin', 'uglifyjs'),
 }
